# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.api_resources.abstract import (
    CreateableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
)
from stripe.api_resources.list_object import ListObject
from stripe.request_options import RequestOptions
from stripe.stripe_object import StripeObject
from typing import ClassVar, List, Optional, Union, cast
from typing_extensions import (
    Literal,
    NotRequired,
    TypedDict,
    Unpack,
    TYPE_CHECKING,
)
from urllib.parse import quote_plus


class Registration(
    CreateableAPIResource["Registration"],
    ListableAPIResource["Registration"],
    UpdateableAPIResource["Registration"],
):
    """
    A Tax `Registration` lets us know that your business is registered to collect tax on payments within a region, enabling you to [automatically collect tax](https://stripe.com/docs/tax).

    Stripe doesn't register on your behalf with the relevant authorities when you create a Tax `Registration` object. For more information on how to register to collect tax, see [our guide](https://stripe.com/docs/tax/registering).

    Related guide: [Using the Registrations API](https://stripe.com/docs/tax/registrations-api)
    """

    OBJECT_NAME: ClassVar[Literal["tax.registration"]] = "tax.registration"

    class CountryOptions(StripeObject):
        class Ae(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class At(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Au(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Be(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Bg(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ca(StripeObject):
            class ProvinceStandard(StripeObject):
                province: str
                """
                Two-letter CA province code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            province_standard: Optional[ProvinceStandard]
            type: Literal["province_standard", "simplified", "standard"]
            """
            Type of registration in Canada.
            """
            _inner_class_types = {"province_standard": ProvinceStandard}

        class Ch(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Cl(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Co(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Cy(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Cz(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class De(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Dk(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ee(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Es(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Fi(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Fr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Gb(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Gr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Hr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Hu(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Id(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ie(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Is(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class It(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Jp(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Kr(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Lt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Lu(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Lv(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Mt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Mx(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class My(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Nl(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class No(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Nz(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Pl(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Pt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ro(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sa(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Se(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sg(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Si(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sk(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["small_seller", "standard"]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Th(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Tr(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Us(StripeObject):
            class LocalAmusementTax(StripeObject):
                jurisdiction: str
                """
                A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction.
                """

            class LocalLeaseTax(StripeObject):
                jurisdiction: str
                """
                A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction.
                """

            local_amusement_tax: Optional[LocalAmusementTax]
            local_lease_tax: Optional[LocalLeaseTax]
            state: str
            """
            Two-letter US state code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """
            type: Literal[
                "local_amusement_tax",
                "local_lease_tax",
                "state_communications_tax",
                "state_sales_tax",
            ]
            """
            Type of registration in the US.
            """
            _inner_class_types = {
                "local_amusement_tax": LocalAmusementTax,
                "local_lease_tax": LocalLeaseTax,
            }

        class Vn(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Za(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        ae: Optional[Ae]
        at: Optional[At]
        au: Optional[Au]
        be: Optional[Be]
        bg: Optional[Bg]
        ca: Optional[Ca]
        ch: Optional[Ch]
        cl: Optional[Cl]
        co: Optional[Co]
        cy: Optional[Cy]
        cz: Optional[Cz]
        de: Optional[De]
        dk: Optional[Dk]
        ee: Optional[Ee]
        es: Optional[Es]
        fi: Optional[Fi]
        fr: Optional[Fr]
        gb: Optional[Gb]
        gr: Optional[Gr]
        hr: Optional[Hr]
        hu: Optional[Hu]
        id: Optional[Id]
        ie: Optional[Ie]
        is_: Optional[Is]
        it: Optional[It]
        jp: Optional[Jp]
        kr: Optional[Kr]
        lt: Optional[Lt]
        lu: Optional[Lu]
        lv: Optional[Lv]
        mt: Optional[Mt]
        mx: Optional[Mx]
        my: Optional[My]
        nl: Optional[Nl]
        no: Optional[No]
        nz: Optional[Nz]
        pl: Optional[Pl]
        pt: Optional[Pt]
        ro: Optional[Ro]
        sa: Optional[Sa]
        se: Optional[Se]
        sg: Optional[Sg]
        si: Optional[Si]
        sk: Optional[Sk]
        th: Optional[Th]
        tr: Optional[Tr]
        us: Optional[Us]
        vn: Optional[Vn]
        za: Optional[Za]
        _inner_class_types = {
            "ae": Ae,
            "at": At,
            "au": Au,
            "be": Be,
            "bg": Bg,
            "ca": Ca,
            "ch": Ch,
            "cl": Cl,
            "co": Co,
            "cy": Cy,
            "cz": Cz,
            "de": De,
            "dk": Dk,
            "ee": Ee,
            "es": Es,
            "fi": Fi,
            "fr": Fr,
            "gb": Gb,
            "gr": Gr,
            "hr": Hr,
            "hu": Hu,
            "id": Id,
            "ie": Ie,
            "is": Is,
            "it": It,
            "jp": Jp,
            "kr": Kr,
            "lt": Lt,
            "lu": Lu,
            "lv": Lv,
            "mt": Mt,
            "mx": Mx,
            "my": My,
            "nl": Nl,
            "no": No,
            "nz": Nz,
            "pl": Pl,
            "pt": Pt,
            "ro": Ro,
            "sa": Sa,
            "se": Se,
            "sg": Sg,
            "si": Si,
            "sk": Sk,
            "th": Th,
            "tr": Tr,
            "us": Us,
            "vn": Vn,
            "za": Za,
        }
        _field_remappings = {"is_": "is"}

    if TYPE_CHECKING:

        class CreateParams(RequestOptions):
            active_from: Union[Literal["now"], int]
            """
            Time at which the Tax Registration becomes active. It can be either `now` to indicate the current time, or a future timestamp measured in seconds since the Unix epoch.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            country_options: "Registration.CreateParamsCountryOptions"
            """
            Specific options for a registration in the specified `country`.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            expires_at: NotRequired["int|None"]
            """
            If set, the Tax Registration stops being active at this time. If not set, the Tax Registration will be active indefinitely. Timestamp measured in seconds since the Unix epoch.
            """

        class CreateParamsCountryOptions(TypedDict):
            ae: NotRequired["Registration.CreateParamsCountryOptionsAe|None"]
            """
            Options for the registration in AE.
            """
            at: NotRequired["Registration.CreateParamsCountryOptionsAt|None"]
            """
            Options for the registration in AT.
            """
            au: NotRequired["Registration.CreateParamsCountryOptionsAu|None"]
            """
            Options for the registration in AU.
            """
            be: NotRequired["Registration.CreateParamsCountryOptionsBe|None"]
            """
            Options for the registration in BE.
            """
            bg: NotRequired["Registration.CreateParamsCountryOptionsBg|None"]
            """
            Options for the registration in BG.
            """
            ca: NotRequired["Registration.CreateParamsCountryOptionsCa|None"]
            """
            Options for the registration in CA.
            """
            ch: NotRequired["Registration.CreateParamsCountryOptionsCh|None"]
            """
            Options for the registration in CH.
            """
            cl: NotRequired["Registration.CreateParamsCountryOptionsCl|None"]
            """
            Options for the registration in CL.
            """
            co: NotRequired["Registration.CreateParamsCountryOptionsCo|None"]
            """
            Options for the registration in CO.
            """
            cy: NotRequired["Registration.CreateParamsCountryOptionsCy|None"]
            """
            Options for the registration in CY.
            """
            cz: NotRequired["Registration.CreateParamsCountryOptionsCz|None"]
            """
            Options for the registration in CZ.
            """
            de: NotRequired["Registration.CreateParamsCountryOptionsDe|None"]
            """
            Options for the registration in DE.
            """
            dk: NotRequired["Registration.CreateParamsCountryOptionsDk|None"]
            """
            Options for the registration in DK.
            """
            ee: NotRequired["Registration.CreateParamsCountryOptionsEe|None"]
            """
            Options for the registration in EE.
            """
            es: NotRequired["Registration.CreateParamsCountryOptionsEs|None"]
            """
            Options for the registration in ES.
            """
            fi: NotRequired["Registration.CreateParamsCountryOptionsFi|None"]
            """
            Options for the registration in FI.
            """
            fr: NotRequired["Registration.CreateParamsCountryOptionsFr|None"]
            """
            Options for the registration in FR.
            """
            gb: NotRequired["Registration.CreateParamsCountryOptionsGb|None"]
            """
            Options for the registration in GB.
            """
            gr: NotRequired["Registration.CreateParamsCountryOptionsGr|None"]
            """
            Options for the registration in GR.
            """
            hr: NotRequired["Registration.CreateParamsCountryOptionsHr|None"]
            """
            Options for the registration in HR.
            """
            hu: NotRequired["Registration.CreateParamsCountryOptionsHu|None"]
            """
            Options for the registration in HU.
            """
            id: NotRequired["Registration.CreateParamsCountryOptionsId|None"]
            """
            Options for the registration in ID.
            """
            ie: NotRequired["Registration.CreateParamsCountryOptionsIe|None"]
            """
            Options for the registration in IE.
            """
            it: NotRequired["Registration.CreateParamsCountryOptionsIt|None"]
            """
            Options for the registration in IT.
            """
            jp: NotRequired["Registration.CreateParamsCountryOptionsJp|None"]
            """
            Options for the registration in JP.
            """
            kr: NotRequired["Registration.CreateParamsCountryOptionsKr|None"]
            """
            Options for the registration in KR.
            """
            lt: NotRequired["Registration.CreateParamsCountryOptionsLt|None"]
            """
            Options for the registration in LT.
            """
            lu: NotRequired["Registration.CreateParamsCountryOptionsLu|None"]
            """
            Options for the registration in LU.
            """
            lv: NotRequired["Registration.CreateParamsCountryOptionsLv|None"]
            """
            Options for the registration in LV.
            """
            mt: NotRequired["Registration.CreateParamsCountryOptionsMt|None"]
            """
            Options for the registration in MT.
            """
            mx: NotRequired["Registration.CreateParamsCountryOptionsMx|None"]
            """
            Options for the registration in MX.
            """
            my: NotRequired["Registration.CreateParamsCountryOptionsMy|None"]
            """
            Options for the registration in MY.
            """
            nl: NotRequired["Registration.CreateParamsCountryOptionsNl|None"]
            """
            Options for the registration in NL.
            """
            no: NotRequired["Registration.CreateParamsCountryOptionsNo|None"]
            """
            Options for the registration in NO.
            """
            nz: NotRequired["Registration.CreateParamsCountryOptionsNz|None"]
            """
            Options for the registration in NZ.
            """
            pl: NotRequired["Registration.CreateParamsCountryOptionsPl|None"]
            """
            Options for the registration in PL.
            """
            pt: NotRequired["Registration.CreateParamsCountryOptionsPt|None"]
            """
            Options for the registration in PT.
            """
            ro: NotRequired["Registration.CreateParamsCountryOptionsRo|None"]
            """
            Options for the registration in RO.
            """
            sa: NotRequired["Registration.CreateParamsCountryOptionsSa|None"]
            """
            Options for the registration in SA.
            """
            se: NotRequired["Registration.CreateParamsCountryOptionsSe|None"]
            """
            Options for the registration in SE.
            """
            sg: NotRequired["Registration.CreateParamsCountryOptionsSg|None"]
            """
            Options for the registration in SG.
            """
            si: NotRequired["Registration.CreateParamsCountryOptionsSi|None"]
            """
            Options for the registration in SI.
            """
            sk: NotRequired["Registration.CreateParamsCountryOptionsSk|None"]
            """
            Options for the registration in SK.
            """
            th: NotRequired["Registration.CreateParamsCountryOptionsTh|None"]
            """
            Options for the registration in TH.
            """
            tr: NotRequired["Registration.CreateParamsCountryOptionsTr|None"]
            """
            Options for the registration in TR.
            """
            us: NotRequired["Registration.CreateParamsCountryOptionsUs|None"]
            """
            Options for the registration in US.
            """
            vn: NotRequired["Registration.CreateParamsCountryOptionsVn|None"]
            """
            Options for the registration in VN.
            """
            za: NotRequired["Registration.CreateParamsCountryOptionsZa|None"]
            """
            Options for the registration in ZA.
            """

        class CreateParamsCountryOptionsZa(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsVn(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsUs(TypedDict):
            local_amusement_tax: NotRequired[
                "Registration.CreateParamsCountryOptionsUsLocalAmusementTax|None"
            ]
            """
            Options for the local amusement tax registration.
            """
            local_lease_tax: NotRequired[
                "Registration.CreateParamsCountryOptionsUsLocalLeaseTax|None"
            ]
            """
            Options for the local lease tax registration.
            """
            state: str
            """
            Two-letter US state code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """
            type: Literal[
                "local_amusement_tax",
                "local_lease_tax",
                "state_communications_tax",
                "state_sales_tax",
            ]
            """
            Type of registration to be created in the US.
            """

        class CreateParamsCountryOptionsUsLocalLeaseTax(TypedDict):
            jurisdiction: str
            """
            A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction. Supported FIPS codes are: `14000` (Chicago).
            """

        class CreateParamsCountryOptionsUsLocalAmusementTax(TypedDict):
            jurisdiction: str
            """
            A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction. Supported FIPS codes are: `14000` (Chicago), `06613` (Bloomington), `21696` (East Dundee), `24582` (Evanston), and `68081` (Schiller Park).
            """

        class CreateParamsCountryOptionsTr(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsTh(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsSk(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsSkStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsSkStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsSi(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsSiStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsSiStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsSg(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsSe(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsSeStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsSeStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsSa(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsRo(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsRoStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsRoStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsPt(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsPtStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsPtStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsPl(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsPlStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsPlStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsNz(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsNo(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsNl(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsNlStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsNlStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsMy(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsMx(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsMt(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsMtStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsMtStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsLv(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsLvStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsLvStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsLu(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsLuStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsLuStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsLt(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsLtStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsLtStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsKr(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsJp(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsIt(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsItStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsItStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsIs(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsIe(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsIeStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsIeStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsId(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsHu(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsHuStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsHuStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsHr(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsHrStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsHrStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsGr(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsGrStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsGrStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsGb(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsFr(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsFrStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsFrStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsFi(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsFiStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsFiStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsEs(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsEsStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsEsStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsEe(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsEeStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsEeStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsDk(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsDkStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsDkStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsDe(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsDeStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsDeStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsCz(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsCzStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsCzStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsCy(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsCyStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsCyStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsCo(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsCl(TypedDict):
            type: Literal["simplified"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsCh(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsCa(TypedDict):
            province_standard: NotRequired[
                "Registration.CreateParamsCountryOptionsCaProvinceStandard|None"
            ]
            """
            Options for the provincial tax registration.
            """
            type: Literal["province_standard", "simplified", "standard"]
            """
            Type of registration to be created in Canada.
            """

        class CreateParamsCountryOptionsCaProvinceStandard(TypedDict):
            province: str
            """
            Two-letter CA province code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """

        class CreateParamsCountryOptionsBg(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsBgStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsBgStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsBe(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsBeStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsBeStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsAu(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class CreateParamsCountryOptionsAt(TypedDict):
            standard: NotRequired[
                "Registration.CreateParamsCountryOptionsAtStandard|None"
            ]
            """
            Options for the standard registration.
            """
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration to be created in an EU country.
            """

        class CreateParamsCountryOptionsAtStandard(TypedDict):
            place_of_supply_scheme: Literal["small_seller", "standard"]
            """
            Place of supply scheme used in an EU standard registration.
            """

        class CreateParamsCountryOptionsAe(TypedDict):
            type: Literal["standard"]
            """
            Type of registration to be created in `country`.
            """

        class ListParams(RequestOptions):
            ending_before: NotRequired["str|None"]
            """
            A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            limit: NotRequired["int|None"]
            """
            A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
            """
            starting_after: NotRequired["str|None"]
            """
            A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
            """
            status: NotRequired[
                "Literal['active', 'all', 'expired', 'scheduled']|None"
            ]
            """
            The status of the Tax Registration.
            """

        class ModifyParams(RequestOptions):
            active_from: NotRequired["Literal['now']|int|None"]
            """
            Time at which the registration becomes active. It can be either `now` to indicate the current time, or a timestamp measured in seconds since the Unix epoch.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            expires_at: NotRequired["Literal['']|Literal['now']|int|None"]
            """
            If set, the registration stops being active at this time. If not set, the registration will be active indefinitely. It can be either `now` to indicate the current time, or a timestamp measured in seconds since the Unix epoch.
            """

    active_from: int
    """
    Time at which the registration becomes active. Measured in seconds since the Unix epoch.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    country_options: CountryOptions
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    expires_at: Optional[int]
    """
    If set, the registration stops being active at this time. If not set, the registration will be active indefinitely. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["tax.registration"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Literal["active", "expired", "scheduled"]
    """
    The status of the registration. This field is present for convenience and can be deduced from `active_from` and `expires_at`.
    """

    @classmethod
    def create(
        cls,
        api_key: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["Registration.CreateParams"]
    ) -> "Registration":
        """
        Creates a new Tax Registration object.
        """
        return cast(
            "Registration",
            cls._static_request(
                "post",
                cls.class_url(),
                api_key,
                idempotency_key,
                stripe_version,
                stripe_account,
                params,
            ),
        )

    @classmethod
    def list(
        cls,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["Registration.ListParams"]
    ) -> ListObject["Registration"]:
        """
        Returns a list of Tax Registration objects.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["Registration.ModifyParams"]
    ) -> "Registration":
        """
        Updates an existing Tax Registration object.

        A registration cannot be deleted after it has been created. If you wish to end a registration you may do so by setting expires_at.
        """
        url = "%s/%s" % (cls.class_url(), quote_plus(id))
        return cast(
            "Registration",
            cls._static_request("post", url, params=params),
        )

    _inner_class_types = {"country_options": CountryOptions}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe import util
from stripe.api_resources.abstract import (
    APIResourceTestHelpers,
    ListableAPIResource,
    UpdateableAPIResource,
)
from stripe.api_resources.expandable_field import ExpandableField
from stripe.api_resources.list_object import ListObject
from stripe.request_options import RequestOptions
from stripe.stripe_object import StripeObject
from stripe.util import class_method_variant
from typing import ClassVar, Dict, List, Optional, cast, overload
from typing_extensions import (
    Literal,
    NotRequired,
    Type,
    TypedDict,
    Unpack,
    TYPE_CHECKING,
)
from urllib.parse import quote_plus

if TYPE_CHECKING:
    from stripe.api_resources.balance_transaction import BalanceTransaction
    from stripe.api_resources.issuing.authorization import Authorization
    from stripe.api_resources.issuing.card import Card
    from stripe.api_resources.issuing.cardholder import Cardholder
    from stripe.api_resources.issuing.dispute import Dispute
    from stripe.api_resources.issuing.token import Token


class Transaction(
    ListableAPIResource["Transaction"],
    UpdateableAPIResource["Transaction"],
):
    """
    Any use of an [issued card](https://stripe.com/docs/issuing) that results in funds entering or leaving
    your Stripe account, such as a completed purchase or refund, is represented by an Issuing
    `Transaction` object.

    Related guide: [Issued card transactions](https://stripe.com/docs/issuing/purchases/transactions)
    """

    OBJECT_NAME: ClassVar[
        Literal["issuing.transaction"]
    ] = "issuing.transaction"

    class AmountDetails(StripeObject):
        atm_fee: Optional[int]
        """
        The fee charged by the ATM for the cash withdrawal.
        """
        cashback_amount: Optional[int]
        """
        The amount of cash requested by the cardholder.
        """

    class MerchantData(StripeObject):
        category: str
        """
        A categorization of the seller's type of business. See our [merchant categories guide](https://stripe.com/docs/issuing/merchant-categories) for a list of possible values.
        """
        category_code: str
        """
        The merchant category code for the seller's business
        """
        city: Optional[str]
        """
        City where the seller is located
        """
        country: Optional[str]
        """
        Country where the seller is located
        """
        name: Optional[str]
        """
        Name of the seller
        """
        network_id: str
        """
        Identifier assigned to the seller by the card network. Different card networks may assign different network_id fields to the same merchant.
        """
        postal_code: Optional[str]
        """
        Postal code where the seller is located
        """
        state: Optional[str]
        """
        State where the seller is located
        """
        terminal_id: Optional[str]
        """
        An ID assigned by the seller to the location of the sale.
        """
        url: Optional[str]
        """
        URL provided by the merchant on a 3DS request
        """

    class PurchaseDetails(StripeObject):
        class Flight(StripeObject):
            class Segment(StripeObject):
                arrival_airport_code: Optional[str]
                """
                The three-letter IATA airport code of the flight's destination.
                """
                carrier: Optional[str]
                """
                The airline carrier code.
                """
                departure_airport_code: Optional[str]
                """
                The three-letter IATA airport code that the flight departed from.
                """
                flight_number: Optional[str]
                """
                The flight number.
                """
                service_class: Optional[str]
                """
                The flight's service class.
                """
                stopover_allowed: Optional[bool]
                """
                Whether a stopover is allowed on this flight.
                """

            departure_at: Optional[int]
            """
            The time that the flight departed.
            """
            passenger_name: Optional[str]
            """
            The name of the passenger.
            """
            refundable: Optional[bool]
            """
            Whether the ticket is refundable.
            """
            segments: Optional[List[Segment]]
            """
            The legs of the trip.
            """
            travel_agency: Optional[str]
            """
            The travel agency that issued the ticket.
            """
            _inner_class_types = {"segments": Segment}

        class Fuel(StripeObject):
            type: str
            """
            The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
            """
            unit: str
            """
            The units for `volume_decimal`. One of `us_gallon` or `liter`.
            """
            unit_cost_decimal: str
            """
            The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
            """
            volume_decimal: Optional[str]
            """
            The volume of the fuel that was pumped, represented as a decimal string with at most 12 decimal places.
            """

        class Lodging(StripeObject):
            check_in_at: Optional[int]
            """
            The time of checking into the lodging.
            """
            nights: Optional[int]
            """
            The number of nights stayed at the lodging.
            """

        class Receipt(StripeObject):
            description: Optional[str]
            """
            The description of the item. The maximum length of this field is 26 characters.
            """
            quantity: Optional[float]
            """
            The quantity of the item.
            """
            total: Optional[int]
            """
            The total for this line item in cents.
            """
            unit_cost: Optional[int]
            """
            The unit cost of the item in cents.
            """

        flight: Optional[Flight]
        """
        Information about the flight that was purchased with this transaction.
        """
        fuel: Optional[Fuel]
        """
        Information about fuel that was purchased with this transaction.
        """
        lodging: Optional[Lodging]
        """
        Information about lodging that was purchased with this transaction.
        """
        receipt: Optional[List[Receipt]]
        """
        The line items in the purchase.
        """
        reference: Optional[str]
        """
        A merchant-specific order number.
        """
        _inner_class_types = {
            "flight": Flight,
            "fuel": Fuel,
            "lodging": Lodging,
            "receipt": Receipt,
        }

    class Treasury(StripeObject):
        received_credit: Optional[str]
        """
        The Treasury [ReceivedCredit](https://stripe.com/docs/api/treasury/received_credits) representing this Issuing transaction if it is a refund
        """
        received_debit: Optional[str]
        """
        The Treasury [ReceivedDebit](https://stripe.com/docs/api/treasury/received_debits) representing this Issuing transaction if it is a capture
        """

    if TYPE_CHECKING:

        class ListParams(RequestOptions):
            card: NotRequired["str|None"]
            """
            Only return transactions that belong to the given card.
            """
            cardholder: NotRequired["str|None"]
            """
            Only return transactions that belong to the given cardholder.
            """
            created: NotRequired["Transaction.ListParamsCreated|int|None"]
            """
            Only return transactions that were created during the given date interval.
            """
            ending_before: NotRequired["str|None"]
            """
            A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            limit: NotRequired["int|None"]
            """
            A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
            """
            starting_after: NotRequired["str|None"]
            """
            A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
            """
            type: NotRequired["Literal['capture', 'refund']|None"]
            """
            Only return transactions that have the given type. One of `capture` or `refund`.
            """

        class ListParamsCreated(TypedDict):
            gt: NotRequired["int|None"]
            """
            Minimum value to filter by (exclusive)
            """
            gte: NotRequired["int|None"]
            """
            Minimum value to filter by (inclusive)
            """
            lt: NotRequired["int|None"]
            """
            Maximum value to filter by (exclusive)
            """
            lte: NotRequired["int|None"]
            """
            Maximum value to filter by (inclusive)
            """

        class ModifyParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            metadata: NotRequired["Literal['']|Dict[str, str]|None"]
            """
            Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
            """

        class RetrieveParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """

        class CreateForceCaptureParams(RequestOptions):
            amount: int
            """
            The total amount to attempt to capture. This amount is in the provided currency, or defaults to the cards currency, and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
            """
            card: str
            """
            Card associated with this transaction.
            """
            currency: NotRequired["str|None"]
            """
            The currency of the capture. If not provided, defaults to the currency of the card. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            merchant_data: NotRequired[
                "Transaction.CreateForceCaptureParamsMerchantData|None"
            ]
            """
            Details about the seller (grocery store, e-commerce website, etc.) where the card authorization happened.
            """
            purchase_details: NotRequired[
                "Transaction.CreateForceCaptureParamsPurchaseDetails|None"
            ]
            """
            Additional purchase information that is optionally provided by the merchant.
            """

        class CreateForceCaptureParamsPurchaseDetails(TypedDict):
            flight: NotRequired[
                "Transaction.CreateForceCaptureParamsPurchaseDetailsFlight|None"
            ]
            """
            Information about the flight that was purchased with this transaction.
            """
            fuel: NotRequired[
                "Transaction.CreateForceCaptureParamsPurchaseDetailsFuel|None"
            ]
            """
            Information about fuel that was purchased with this transaction.
            """
            lodging: NotRequired[
                "Transaction.CreateForceCaptureParamsPurchaseDetailsLodging|None"
            ]
            """
            Information about lodging that was purchased with this transaction.
            """
            receipt: NotRequired[
                "List[Transaction.CreateForceCaptureParamsPurchaseDetailsReceipt]|None"
            ]
            """
            The line items in the purchase.
            """
            reference: NotRequired["str|None"]
            """
            A merchant-specific order number.
            """

        class CreateForceCaptureParamsPurchaseDetailsReceipt(TypedDict):
            description: NotRequired["str|None"]
            quantity: NotRequired["str|None"]
            total: NotRequired["int|None"]
            unit_cost: NotRequired["int|None"]

        class CreateForceCaptureParamsPurchaseDetailsLodging(TypedDict):
            check_in_at: NotRequired["int|None"]
            """
            The time of checking into the lodging.
            """
            nights: NotRequired["int|None"]
            """
            The number of nights stayed at the lodging.
            """

        class CreateForceCaptureParamsPurchaseDetailsFuel(TypedDict):
            type: NotRequired[
                "Literal['diesel', 'other', 'unleaded_plus', 'unleaded_regular', 'unleaded_super']|None"
            ]
            """
            The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
            """
            unit: NotRequired["Literal['liter', 'us_gallon']|None"]
            """
            The units for `volume_decimal`. One of `us_gallon` or `liter`.
            """
            unit_cost_decimal: NotRequired["str|None"]
            """
            The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
            """
            volume_decimal: NotRequired["str|None"]
            """
            The volume of the fuel that was pumped, represented as a decimal string with at most 12 decimal places.
            """

        class CreateForceCaptureParamsPurchaseDetailsFlight(TypedDict):
            departure_at: NotRequired["int|None"]
            """
            The time that the flight departed.
            """
            passenger_name: NotRequired["str|None"]
            """
            The name of the passenger.
            """
            refundable: NotRequired["bool|None"]
            """
            Whether the ticket is refundable.
            """
            segments: NotRequired[
                "List[Transaction.CreateForceCaptureParamsPurchaseDetailsFlightSegment]|None"
            ]
            """
            The legs of the trip.
            """
            travel_agency: NotRequired["str|None"]
            """
            The travel agency that issued the ticket.
            """

        class CreateForceCaptureParamsPurchaseDetailsFlightSegment(TypedDict):
            arrival_airport_code: NotRequired["str|None"]
            """
            The three-letter IATA airport code of the flight's destination.
            """
            carrier: NotRequired["str|None"]
            """
            The airline carrier code.
            """
            departure_airport_code: NotRequired["str|None"]
            """
            The three-letter IATA airport code that the flight departed from.
            """
            flight_number: NotRequired["str|None"]
            """
            The flight number.
            """
            service_class: NotRequired["str|None"]
            """
            The flight's service class.
            """
            stopover_allowed: NotRequired["bool|None"]
            """
            Whether a stopover is allowed on this flight.
            """

        class CreateForceCaptureParamsMerchantData(TypedDict):
            category: NotRequired[
                "Literal['ac_refrigeration_repair', 'accounting_bookkeeping_services', 'advertising_services', 'agricultural_cooperative', 'airlines_air_carriers', 'airports_flying_fields', 'ambulance_services', 'amusement_parks_carnivals', 'antique_reproductions', 'antique_shops', 'aquariums', 'architectural_surveying_services', 'art_dealers_and_galleries', 'artists_supply_and_craft_shops', 'auto_and_home_supply_stores', 'auto_body_repair_shops', 'auto_paint_shops', 'auto_service_shops', 'automated_cash_disburse', 'automated_fuel_dispensers', 'automobile_associations', 'automotive_parts_and_accessories_stores', 'automotive_tire_stores', 'bail_and_bond_payments', 'bakeries', 'bands_orchestras', 'barber_and_beauty_shops', 'betting_casino_gambling', 'bicycle_shops', 'billiard_pool_establishments', 'boat_dealers', 'boat_rentals_and_leases', 'book_stores', 'books_periodicals_and_newspapers', 'bowling_alleys', 'bus_lines', 'business_secretarial_schools', 'buying_shopping_services', 'cable_satellite_and_other_pay_television_and_radio', 'camera_and_photographic_supply_stores', 'candy_nut_and_confectionery_stores', 'car_and_truck_dealers_new_used', 'car_and_truck_dealers_used_only', 'car_rental_agencies', 'car_washes', 'carpentry_services', 'carpet_upholstery_cleaning', 'caterers', 'charitable_and_social_service_organizations_fundraising', 'chemicals_and_allied_products', 'child_care_services', 'childrens_and_infants_wear_stores', 'chiropodists_podiatrists', 'chiropractors', 'cigar_stores_and_stands', 'civic_social_fraternal_associations', 'cleaning_and_maintenance', 'clothing_rental', 'colleges_universities', 'commercial_equipment', 'commercial_footwear', 'commercial_photography_art_and_graphics', 'commuter_transport_and_ferries', 'computer_network_services', 'computer_programming', 'computer_repair', 'computer_software_stores', 'computers_peripherals_and_software', 'concrete_work_services', 'construction_materials', 'consulting_public_relations', 'correspondence_schools', 'cosmetic_stores', 'counseling_services', 'country_clubs', 'courier_services', 'court_costs', 'credit_reporting_agencies', 'cruise_lines', 'dairy_products_stores', 'dance_hall_studios_schools', 'dating_escort_services', 'dentists_orthodontists', 'department_stores', 'detective_agencies', 'digital_goods_applications', 'digital_goods_games', 'digital_goods_large_volume', 'digital_goods_media', 'direct_marketing_catalog_merchant', 'direct_marketing_combination_catalog_and_retail_merchant', 'direct_marketing_inbound_telemarketing', 'direct_marketing_insurance_services', 'direct_marketing_other', 'direct_marketing_outbound_telemarketing', 'direct_marketing_subscription', 'direct_marketing_travel', 'discount_stores', 'doctors', 'door_to_door_sales', 'drapery_window_covering_and_upholstery_stores', 'drinking_places', 'drug_stores_and_pharmacies', 'drugs_drug_proprietaries_and_druggist_sundries', 'dry_cleaners', 'durable_goods', 'duty_free_stores', 'eating_places_restaurants', 'educational_services', 'electric_razor_stores', 'electric_vehicle_charging', 'electrical_parts_and_equipment', 'electrical_services', 'electronics_repair_shops', 'electronics_stores', 'elementary_secondary_schools', 'emergency_services_gcas_visa_use_only', 'employment_temp_agencies', 'equipment_rental', 'exterminating_services', 'family_clothing_stores', 'fast_food_restaurants', 'financial_institutions', 'fines_government_administrative_entities', 'fireplace_fireplace_screens_and_accessories_stores', 'floor_covering_stores', 'florists', 'florists_supplies_nursery_stock_and_flowers', 'freezer_and_locker_meat_provisioners', 'fuel_dealers_non_automotive', 'funeral_services_crematories', 'furniture_home_furnishings_and_equipment_stores_except_appliances', 'furniture_repair_refinishing', 'furriers_and_fur_shops', 'general_services', 'gift_card_novelty_and_souvenir_shops', 'glass_paint_and_wallpaper_stores', 'glassware_crystal_stores', 'golf_courses_public', 'government_licensed_horse_dog_racing_us_region_only', 'government_licensed_online_casions_online_gambling_us_region_only', 'government_owned_lotteries_non_us_region', 'government_owned_lotteries_us_region_only', 'government_services', 'grocery_stores_supermarkets', 'hardware_equipment_and_supplies', 'hardware_stores', 'health_and_beauty_spas', 'hearing_aids_sales_and_supplies', 'heating_plumbing_a_c', 'hobby_toy_and_game_shops', 'home_supply_warehouse_stores', 'hospitals', 'hotels_motels_and_resorts', 'household_appliance_stores', 'industrial_supplies', 'information_retrieval_services', 'insurance_default', 'insurance_underwriting_premiums', 'intra_company_purchases', 'jewelry_stores_watches_clocks_and_silverware_stores', 'landscaping_services', 'laundries', 'laundry_cleaning_services', 'legal_services_attorneys', 'luggage_and_leather_goods_stores', 'lumber_building_materials_stores', 'manual_cash_disburse', 'marinas_service_and_supplies', 'marketplaces', 'masonry_stonework_and_plaster', 'massage_parlors', 'medical_and_dental_labs', 'medical_dental_ophthalmic_and_hospital_equipment_and_supplies', 'medical_services', 'membership_organizations', 'mens_and_boys_clothing_and_accessories_stores', 'mens_womens_clothing_stores', 'metal_service_centers', 'miscellaneous_apparel_and_accessory_shops', 'miscellaneous_auto_dealers', 'miscellaneous_business_services', 'miscellaneous_food_stores', 'miscellaneous_general_merchandise', 'miscellaneous_general_services', 'miscellaneous_home_furnishing_specialty_stores', 'miscellaneous_publishing_and_printing', 'miscellaneous_recreation_services', 'miscellaneous_repair_shops', 'miscellaneous_specialty_retail', 'mobile_home_dealers', 'motion_picture_theaters', 'motor_freight_carriers_and_trucking', 'motor_homes_dealers', 'motor_vehicle_supplies_and_new_parts', 'motorcycle_shops_and_dealers', 'motorcycle_shops_dealers', 'music_stores_musical_instruments_pianos_and_sheet_music', 'news_dealers_and_newsstands', 'non_fi_money_orders', 'non_fi_stored_value_card_purchase_load', 'nondurable_goods', 'nurseries_lawn_and_garden_supply_stores', 'nursing_personal_care', 'office_and_commercial_furniture', 'opticians_eyeglasses', 'optometrists_ophthalmologist', 'orthopedic_goods_prosthetic_devices', 'osteopaths', 'package_stores_beer_wine_and_liquor', 'paints_varnishes_and_supplies', 'parking_lots_garages', 'passenger_railways', 'pawn_shops', 'pet_shops_pet_food_and_supplies', 'petroleum_and_petroleum_products', 'photo_developing', 'photographic_photocopy_microfilm_equipment_and_supplies', 'photographic_studios', 'picture_video_production', 'piece_goods_notions_and_other_dry_goods', 'plumbing_heating_equipment_and_supplies', 'political_organizations', 'postal_services_government_only', 'precious_stones_and_metals_watches_and_jewelry', 'professional_services', 'public_warehousing_and_storage', 'quick_copy_repro_and_blueprint', 'railroads', 'real_estate_agents_and_managers_rentals', 'record_stores', 'recreational_vehicle_rentals', 'religious_goods_stores', 'religious_organizations', 'roofing_siding_sheet_metal', 'secretarial_support_services', 'security_brokers_dealers', 'service_stations', 'sewing_needlework_fabric_and_piece_goods_stores', 'shoe_repair_hat_cleaning', 'shoe_stores', 'small_appliance_repair', 'snowmobile_dealers', 'special_trade_services', 'specialty_cleaning', 'sporting_goods_stores', 'sporting_recreation_camps', 'sports_and_riding_apparel_stores', 'sports_clubs_fields', 'stamp_and_coin_stores', 'stationary_office_supplies_printing_and_writing_paper', 'stationery_stores_office_and_school_supply_stores', 'swimming_pools_sales', 't_ui_travel_germany', 'tailors_alterations', 'tax_payments_government_agencies', 'tax_preparation_services', 'taxicabs_limousines', 'telecommunication_equipment_and_telephone_sales', 'telecommunication_services', 'telegraph_services', 'tent_and_awning_shops', 'testing_laboratories', 'theatrical_ticket_agencies', 'timeshares', 'tire_retreading_and_repair', 'tolls_bridge_fees', 'tourist_attractions_and_exhibits', 'towing_services', 'trailer_parks_campgrounds', 'transportation_services', 'travel_agencies_tour_operators', 'truck_stop_iteration', 'truck_utility_trailer_rentals', 'typesetting_plate_making_and_related_services', 'typewriter_stores', 'u_s_federal_government_agencies_or_departments', 'uniforms_commercial_clothing', 'used_merchandise_and_secondhand_stores', 'utilities', 'variety_stores', 'veterinary_services', 'video_amusement_game_supplies', 'video_game_arcades', 'video_tape_rental_stores', 'vocational_trade_schools', 'watch_jewelry_repair', 'welding_repair', 'wholesale_clubs', 'wig_and_toupee_stores', 'wires_money_orders', 'womens_accessory_and_specialty_shops', 'womens_ready_to_wear_stores', 'wrecking_and_salvage_yards']|None"
            ]
            """
            A categorization of the seller's type of business. See our [merchant categories guide](https://stripe.com/docs/issuing/merchant-categories) for a list of possible values.
            """
            city: NotRequired["str|None"]
            """
            City where the seller is located
            """
            country: NotRequired["str|None"]
            """
            Country where the seller is located
            """
            name: NotRequired["str|None"]
            """
            Name of the seller
            """
            network_id: NotRequired["str|None"]
            """
            Identifier assigned to the seller by the card network. Different card networks may assign different network_id fields to the same merchant.
            """
            postal_code: NotRequired["str|None"]
            """
            Postal code where the seller is located
            """
            state: NotRequired["str|None"]
            """
            State where the seller is located
            """
            terminal_id: NotRequired["str|None"]
            """
            An ID assigned by the seller to the location of the sale.
            """
            url: NotRequired["str|None"]
            """
            URL provided by the merchant on a 3DS request
            """

        class CreateUnlinkedRefundParams(RequestOptions):
            amount: int
            """
            The total amount to attempt to refund. This amount is in the provided currency, or defaults to the cards currency, and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
            """
            card: str
            """
            Card associated with this unlinked refund transaction.
            """
            currency: NotRequired["str|None"]
            """
            The currency of the unlinked refund. If not provided, defaults to the currency of the card. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            merchant_data: NotRequired[
                "Transaction.CreateUnlinkedRefundParamsMerchantData|None"
            ]
            """
            Details about the seller (grocery store, e-commerce website, etc.) where the card authorization happened.
            """
            purchase_details: NotRequired[
                "Transaction.CreateUnlinkedRefundParamsPurchaseDetails|None"
            ]
            """
            Additional purchase information that is optionally provided by the merchant.
            """

        class CreateUnlinkedRefundParamsPurchaseDetails(TypedDict):
            flight: NotRequired[
                "Transaction.CreateUnlinkedRefundParamsPurchaseDetailsFlight|None"
            ]
            """
            Information about the flight that was purchased with this transaction.
            """
            fuel: NotRequired[
                "Transaction.CreateUnlinkedRefundParamsPurchaseDetailsFuel|None"
            ]
            """
            Information about fuel that was purchased with this transaction.
            """
            lodging: NotRequired[
                "Transaction.CreateUnlinkedRefundParamsPurchaseDetailsLodging|None"
            ]
            """
            Information about lodging that was purchased with this transaction.
            """
            receipt: NotRequired[
                "List[Transaction.CreateUnlinkedRefundParamsPurchaseDetailsReceipt]|None"
            ]
            """
            The line items in the purchase.
            """
            reference: NotRequired["str|None"]
            """
            A merchant-specific order number.
            """

        class CreateUnlinkedRefundParamsPurchaseDetailsReceipt(TypedDict):
            description: NotRequired["str|None"]
            quantity: NotRequired["str|None"]
            total: NotRequired["int|None"]
            unit_cost: NotRequired["int|None"]

        class CreateUnlinkedRefundParamsPurchaseDetailsLodging(TypedDict):
            check_in_at: NotRequired["int|None"]
            """
            The time of checking into the lodging.
            """
            nights: NotRequired["int|None"]
            """
            The number of nights stayed at the lodging.
            """

        class CreateUnlinkedRefundParamsPurchaseDetailsFuel(TypedDict):
            type: NotRequired[
                "Literal['diesel', 'other', 'unleaded_plus', 'unleaded_regular', 'unleaded_super']|None"
            ]
            """
            The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
            """
            unit: NotRequired["Literal['liter', 'us_gallon']|None"]
            """
            The units for `volume_decimal`. One of `us_gallon` or `liter`.
            """
            unit_cost_decimal: NotRequired["str|None"]
            """
            The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
            """
            volume_decimal: NotRequired["str|None"]
            """
            The volume of the fuel that was pumped, represented as a decimal string with at most 12 decimal places.
            """

        class CreateUnlinkedRefundParamsPurchaseDetailsFlight(TypedDict):
            departure_at: NotRequired["int|None"]
            """
            The time that the flight departed.
            """
            passenger_name: NotRequired["str|None"]
            """
            The name of the passenger.
            """
            refundable: NotRequired["bool|None"]
            """
            Whether the ticket is refundable.
            """
            segments: NotRequired[
                "List[Transaction.CreateUnlinkedRefundParamsPurchaseDetailsFlightSegment]|None"
            ]
            """
            The legs of the trip.
            """
            travel_agency: NotRequired["str|None"]
            """
            The travel agency that issued the ticket.
            """

        class CreateUnlinkedRefundParamsPurchaseDetailsFlightSegment(
            TypedDict
        ):
            arrival_airport_code: NotRequired["str|None"]
            """
            The three-letter IATA airport code of the flight's destination.
            """
            carrier: NotRequired["str|None"]
            """
            The airline carrier code.
            """
            departure_airport_code: NotRequired["str|None"]
            """
            The three-letter IATA airport code that the flight departed from.
            """
            flight_number: NotRequired["str|None"]
            """
            The flight number.
            """
            service_class: NotRequired["str|None"]
            """
            The flight's service class.
            """
            stopover_allowed: NotRequired["bool|None"]
            """
            Whether a stopover is allowed on this flight.
            """

        class CreateUnlinkedRefundParamsMerchantData(TypedDict):
            category: NotRequired[
                "Literal['ac_refrigeration_repair', 'accounting_bookkeeping_services', 'advertising_services', 'agricultural_cooperative', 'airlines_air_carriers', 'airports_flying_fields', 'ambulance_services', 'amusement_parks_carnivals', 'antique_reproductions', 'antique_shops', 'aquariums', 'architectural_surveying_services', 'art_dealers_and_galleries', 'artists_supply_and_craft_shops', 'auto_and_home_supply_stores', 'auto_body_repair_shops', 'auto_paint_shops', 'auto_service_shops', 'automated_cash_disburse', 'automated_fuel_dispensers', 'automobile_associations', 'automotive_parts_and_accessories_stores', 'automotive_tire_stores', 'bail_and_bond_payments', 'bakeries', 'bands_orchestras', 'barber_and_beauty_shops', 'betting_casino_gambling', 'bicycle_shops', 'billiard_pool_establishments', 'boat_dealers', 'boat_rentals_and_leases', 'book_stores', 'books_periodicals_and_newspapers', 'bowling_alleys', 'bus_lines', 'business_secretarial_schools', 'buying_shopping_services', 'cable_satellite_and_other_pay_television_and_radio', 'camera_and_photographic_supply_stores', 'candy_nut_and_confectionery_stores', 'car_and_truck_dealers_new_used', 'car_and_truck_dealers_used_only', 'car_rental_agencies', 'car_washes', 'carpentry_services', 'carpet_upholstery_cleaning', 'caterers', 'charitable_and_social_service_organizations_fundraising', 'chemicals_and_allied_products', 'child_care_services', 'childrens_and_infants_wear_stores', 'chiropodists_podiatrists', 'chiropractors', 'cigar_stores_and_stands', 'civic_social_fraternal_associations', 'cleaning_and_maintenance', 'clothing_rental', 'colleges_universities', 'commercial_equipment', 'commercial_footwear', 'commercial_photography_art_and_graphics', 'commuter_transport_and_ferries', 'computer_network_services', 'computer_programming', 'computer_repair', 'computer_software_stores', 'computers_peripherals_and_software', 'concrete_work_services', 'construction_materials', 'consulting_public_relations', 'correspondence_schools', 'cosmetic_stores', 'counseling_services', 'country_clubs', 'courier_services', 'court_costs', 'credit_reporting_agencies', 'cruise_lines', 'dairy_products_stores', 'dance_hall_studios_schools', 'dating_escort_services', 'dentists_orthodontists', 'department_stores', 'detective_agencies', 'digital_goods_applications', 'digital_goods_games', 'digital_goods_large_volume', 'digital_goods_media', 'direct_marketing_catalog_merchant', 'direct_marketing_combination_catalog_and_retail_merchant', 'direct_marketing_inbound_telemarketing', 'direct_marketing_insurance_services', 'direct_marketing_other', 'direct_marketing_outbound_telemarketing', 'direct_marketing_subscription', 'direct_marketing_travel', 'discount_stores', 'doctors', 'door_to_door_sales', 'drapery_window_covering_and_upholstery_stores', 'drinking_places', 'drug_stores_and_pharmacies', 'drugs_drug_proprietaries_and_druggist_sundries', 'dry_cleaners', 'durable_goods', 'duty_free_stores', 'eating_places_restaurants', 'educational_services', 'electric_razor_stores', 'electric_vehicle_charging', 'electrical_parts_and_equipment', 'electrical_services', 'electronics_repair_shops', 'electronics_stores', 'elementary_secondary_schools', 'emergency_services_gcas_visa_use_only', 'employment_temp_agencies', 'equipment_rental', 'exterminating_services', 'family_clothing_stores', 'fast_food_restaurants', 'financial_institutions', 'fines_government_administrative_entities', 'fireplace_fireplace_screens_and_accessories_stores', 'floor_covering_stores', 'florists', 'florists_supplies_nursery_stock_and_flowers', 'freezer_and_locker_meat_provisioners', 'fuel_dealers_non_automotive', 'funeral_services_crematories', 'furniture_home_furnishings_and_equipment_stores_except_appliances', 'furniture_repair_refinishing', 'furriers_and_fur_shops', 'general_services', 'gift_card_novelty_and_souvenir_shops', 'glass_paint_and_wallpaper_stores', 'glassware_crystal_stores', 'golf_courses_public', 'government_licensed_horse_dog_racing_us_region_only', 'government_licensed_online_casions_online_gambling_us_region_only', 'government_owned_lotteries_non_us_region', 'government_owned_lotteries_us_region_only', 'government_services', 'grocery_stores_supermarkets', 'hardware_equipment_and_supplies', 'hardware_stores', 'health_and_beauty_spas', 'hearing_aids_sales_and_supplies', 'heating_plumbing_a_c', 'hobby_toy_and_game_shops', 'home_supply_warehouse_stores', 'hospitals', 'hotels_motels_and_resorts', 'household_appliance_stores', 'industrial_supplies', 'information_retrieval_services', 'insurance_default', 'insurance_underwriting_premiums', 'intra_company_purchases', 'jewelry_stores_watches_clocks_and_silverware_stores', 'landscaping_services', 'laundries', 'laundry_cleaning_services', 'legal_services_attorneys', 'luggage_and_leather_goods_stores', 'lumber_building_materials_stores', 'manual_cash_disburse', 'marinas_service_and_supplies', 'marketplaces', 'masonry_stonework_and_plaster', 'massage_parlors', 'medical_and_dental_labs', 'medical_dental_ophthalmic_and_hospital_equipment_and_supplies', 'medical_services', 'membership_organizations', 'mens_and_boys_clothing_and_accessories_stores', 'mens_womens_clothing_stores', 'metal_service_centers', 'miscellaneous_apparel_and_accessory_shops', 'miscellaneous_auto_dealers', 'miscellaneous_business_services', 'miscellaneous_food_stores', 'miscellaneous_general_merchandise', 'miscellaneous_general_services', 'miscellaneous_home_furnishing_specialty_stores', 'miscellaneous_publishing_and_printing', 'miscellaneous_recreation_services', 'miscellaneous_repair_shops', 'miscellaneous_specialty_retail', 'mobile_home_dealers', 'motion_picture_theaters', 'motor_freight_carriers_and_trucking', 'motor_homes_dealers', 'motor_vehicle_supplies_and_new_parts', 'motorcycle_shops_and_dealers', 'motorcycle_shops_dealers', 'music_stores_musical_instruments_pianos_and_sheet_music', 'news_dealers_and_newsstands', 'non_fi_money_orders', 'non_fi_stored_value_card_purchase_load', 'nondurable_goods', 'nurseries_lawn_and_garden_supply_stores', 'nursing_personal_care', 'office_and_commercial_furniture', 'opticians_eyeglasses', 'optometrists_ophthalmologist', 'orthopedic_goods_prosthetic_devices', 'osteopaths', 'package_stores_beer_wine_and_liquor', 'paints_varnishes_and_supplies', 'parking_lots_garages', 'passenger_railways', 'pawn_shops', 'pet_shops_pet_food_and_supplies', 'petroleum_and_petroleum_products', 'photo_developing', 'photographic_photocopy_microfilm_equipment_and_supplies', 'photographic_studios', 'picture_video_production', 'piece_goods_notions_and_other_dry_goods', 'plumbing_heating_equipment_and_supplies', 'political_organizations', 'postal_services_government_only', 'precious_stones_and_metals_watches_and_jewelry', 'professional_services', 'public_warehousing_and_storage', 'quick_copy_repro_and_blueprint', 'railroads', 'real_estate_agents_and_managers_rentals', 'record_stores', 'recreational_vehicle_rentals', 'religious_goods_stores', 'religious_organizations', 'roofing_siding_sheet_metal', 'secretarial_support_services', 'security_brokers_dealers', 'service_stations', 'sewing_needlework_fabric_and_piece_goods_stores', 'shoe_repair_hat_cleaning', 'shoe_stores', 'small_appliance_repair', 'snowmobile_dealers', 'special_trade_services', 'specialty_cleaning', 'sporting_goods_stores', 'sporting_recreation_camps', 'sports_and_riding_apparel_stores', 'sports_clubs_fields', 'stamp_and_coin_stores', 'stationary_office_supplies_printing_and_writing_paper', 'stationery_stores_office_and_school_supply_stores', 'swimming_pools_sales', 't_ui_travel_germany', 'tailors_alterations', 'tax_payments_government_agencies', 'tax_preparation_services', 'taxicabs_limousines', 'telecommunication_equipment_and_telephone_sales', 'telecommunication_services', 'telegraph_services', 'tent_and_awning_shops', 'testing_laboratories', 'theatrical_ticket_agencies', 'timeshares', 'tire_retreading_and_repair', 'tolls_bridge_fees', 'tourist_attractions_and_exhibits', 'towing_services', 'trailer_parks_campgrounds', 'transportation_services', 'travel_agencies_tour_operators', 'truck_stop_iteration', 'truck_utility_trailer_rentals', 'typesetting_plate_making_and_related_services', 'typewriter_stores', 'u_s_federal_government_agencies_or_departments', 'uniforms_commercial_clothing', 'used_merchandise_and_secondhand_stores', 'utilities', 'variety_stores', 'veterinary_services', 'video_amusement_game_supplies', 'video_game_arcades', 'video_tape_rental_stores', 'vocational_trade_schools', 'watch_jewelry_repair', 'welding_repair', 'wholesale_clubs', 'wig_and_toupee_stores', 'wires_money_orders', 'womens_accessory_and_specialty_shops', 'womens_ready_to_wear_stores', 'wrecking_and_salvage_yards']|None"
            ]
            """
            A categorization of the seller's type of business. See our [merchant categories guide](https://stripe.com/docs/issuing/merchant-categories) for a list of possible values.
            """
            city: NotRequired["str|None"]
            """
            City where the seller is located
            """
            country: NotRequired["str|None"]
            """
            Country where the seller is located
            """
            name: NotRequired["str|None"]
            """
            Name of the seller
            """
            network_id: NotRequired["str|None"]
            """
            Identifier assigned to the seller by the card network. Different card networks may assign different network_id fields to the same merchant.
            """
            postal_code: NotRequired["str|None"]
            """
            Postal code where the seller is located
            """
            state: NotRequired["str|None"]
            """
            State where the seller is located
            """
            terminal_id: NotRequired["str|None"]
            """
            An ID assigned by the seller to the location of the sale.
            """
            url: NotRequired["str|None"]
            """
            URL provided by the merchant on a 3DS request
            """

        class RefundParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            refund_amount: NotRequired["int|None"]
            """
            The total amount to attempt to refund. This amount is in the provided currency, or defaults to the cards currency, and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
            """

    amount: int
    """
    The transaction amount, which will be reflected in your balance. This amount is in your currency and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
    """
    amount_details: Optional[AmountDetails]
    """
    Detailed breakdown of amount components. These amounts are denominated in `currency` and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
    """
    authorization: Optional[ExpandableField["Authorization"]]
    """
    The `Authorization` object that led to this transaction.
    """
    balance_transaction: Optional[ExpandableField["BalanceTransaction"]]
    """
    ID of the [balance transaction](https://stripe.com/docs/api/balance_transactions) associated with this transaction.
    """
    card: ExpandableField["Card"]
    """
    The card used to make this transaction.
    """
    cardholder: Optional[ExpandableField["Cardholder"]]
    """
    The cardholder to whom this transaction belongs.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    dispute: Optional[ExpandableField["Dispute"]]
    """
    If you've disputed the transaction, the ID of the dispute.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    merchant_amount: int
    """
    The amount that the merchant will receive, denominated in `merchant_currency` and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal). It will be different from `amount` if the merchant is taking payment in a different currency.
    """
    merchant_currency: str
    """
    The currency with which the merchant is taking payment.
    """
    merchant_data: MerchantData
    metadata: Dict[str, str]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["issuing.transaction"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    purchase_details: Optional[PurchaseDetails]
    """
    Additional purchase information that is optionally provided by the merchant.
    """
    token: Optional[ExpandableField["Token"]]
    """
    [Token](https://stripe.com/docs/api/issuing/tokens/object) object used for this transaction. If a network token was not used for this transaction, this field will be null.
    """
    treasury: Optional[Treasury]
    """
    [Treasury](https://stripe.com/docs/api/treasury) details related to this transaction if it was created on a [FinancialAccount](/docs/api/treasury/financial_accounts
    """
    type: Literal["capture", "refund"]
    """
    The nature of the transaction.
    """
    wallet: Optional[Literal["apple_pay", "google_pay", "samsung_pay"]]
    """
    The digital wallet used for this transaction. One of `apple_pay`, `google_pay`, or `samsung_pay`.
    """

    @classmethod
    def list(
        cls,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["Transaction.ListParams"]
    ) -> ListObject["Transaction"]:
        """
        Returns a list of Issuing Transaction objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["Transaction.ModifyParams"]
    ) -> "Transaction":
        """
        Updates the specified Issuing Transaction object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        url = "%s/%s" % (cls.class_url(), quote_plus(id))
        return cast(
            "Transaction",
            cls._static_request("post", url, params=params),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["Transaction.RetrieveParams"]
    ) -> "Transaction":
        """
        Retrieves an Issuing Transaction object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    class TestHelpers(APIResourceTestHelpers["Transaction"]):
        _resource_cls: Type["Transaction"]

        @classmethod
        def create_force_capture(
            cls,
            api_key: Optional[str] = None,
            stripe_version: Optional[str] = None,
            stripe_account: Optional[str] = None,
            **params: Unpack["Transaction.CreateForceCaptureParams"]
        ) -> "Transaction":
            """
            Allows the user to capture an arbitrary amount, also known as a forced capture.
            """
            return cast(
                "Transaction",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/transactions/create_force_capture",
                    api_key=api_key,
                    stripe_version=stripe_version,
                    stripe_account=stripe_account,
                    params=params,
                ),
            )

        @classmethod
        def create_unlinked_refund(
            cls,
            api_key: Optional[str] = None,
            stripe_version: Optional[str] = None,
            stripe_account: Optional[str] = None,
            **params: Unpack["Transaction.CreateUnlinkedRefundParams"]
        ) -> "Transaction":
            """
            Allows the user to refund an arbitrary amount, also known as a unlinked refund.
            """
            return cast(
                "Transaction",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/transactions/create_unlinked_refund",
                    api_key=api_key,
                    stripe_version=stripe_version,
                    stripe_account=stripe_account,
                    params=params,
                ),
            )

        @classmethod
        def _cls_refund(
            cls,
            transaction: str,
            api_key: Optional[str] = None,
            stripe_version: Optional[str] = None,
            stripe_account: Optional[str] = None,
            **params: Unpack["Transaction.RefundParams"]
        ) -> "Transaction":
            """
            Refund a test-mode Transaction.
            """
            return cast(
                "Transaction",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/transactions/{transaction}/refund".format(
                        transaction=util.sanitize_id(transaction)
                    ),
                    api_key=api_key,
                    stripe_version=stripe_version,
                    stripe_account=stripe_account,
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def refund(
            transaction: str,
            api_key: Optional[str] = None,
            stripe_version: Optional[str] = None,
            stripe_account: Optional[str] = None,
            **params: Unpack["Transaction.RefundParams"]
        ) -> "Transaction":
            """
            Refund a test-mode Transaction.
            """
            ...

        @overload
        def refund(
            self,
            idempotency_key: Optional[str] = None,
            **params: Unpack["Transaction.RefundParams"]
        ) -> "Transaction":
            """
            Refund a test-mode Transaction.
            """
            ...

        @class_method_variant("_cls_refund")
        def refund(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            idempotency_key: Optional[str] = None,
            **params: Unpack["Transaction.RefundParams"]
        ) -> "Transaction":
            """
            Refund a test-mode Transaction.
            """
            return cast(
                "Transaction",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/transactions/{transaction}/refund".format(
                        transaction=util.sanitize_id(self.resource.get("id"))
                    ),
                    idempotency_key=idempotency_key,
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "amount_details": AmountDetails,
        "merchant_data": MerchantData,
        "purchase_details": PurchaseDetails,
        "treasury": Treasury,
    }


Transaction.TestHelpers._resource_cls = Transaction



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe import util
from stripe.api_resources.abstract import (
    CreateableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
)
from stripe.api_resources.list_object import ListObject
from stripe.request_options import RequestOptions
from stripe.stripe_object import StripeObject
from stripe.util import class_method_variant
from typing import ClassVar, Dict, List, Optional, cast, overload
from typing_extensions import (
    Literal,
    NotRequired,
    TypedDict,
    Unpack,
    TYPE_CHECKING,
)
from urllib.parse import quote_plus

if TYPE_CHECKING:
    from stripe.api_resources.treasury.financial_account_features import (
        FinancialAccountFeatures,
    )


class FinancialAccount(
    CreateableAPIResource["FinancialAccount"],
    ListableAPIResource["FinancialAccount"],
    UpdateableAPIResource["FinancialAccount"],
):
    """
    Stripe Treasury provides users with a container for money called a FinancialAccount that is separate from their Payments balance.
    FinancialAccounts serve as the source and destination of Treasury's money movement APIs.
    """

    OBJECT_NAME: ClassVar[
        Literal["treasury.financial_account"]
    ] = "treasury.financial_account"

    class Balance(StripeObject):
        cash: Dict[str, int]
        """
        Funds the user can spend right now.
        """
        inbound_pending: Dict[str, int]
        """
        Funds not spendable yet, but will become available at a later time.
        """
        outbound_pending: Dict[str, int]
        """
        Funds in the account, but not spendable because they are being held for pending outbound flows.
        """

    class FinancialAddress(StripeObject):
        class Aba(StripeObject):
            account_holder_name: str
            """
            The name of the person or business that owns the bank account.
            """
            account_number: Optional[str]
            """
            The account number.
            """
            account_number_last4: str
            """
            The last four characters of the account number.
            """
            bank_name: str
            """
            Name of the bank.
            """
            routing_number: str
            """
            Routing number for the account.
            """

        aba: Optional[Aba]
        """
        ABA Records contain U.S. bank account details per the ABA format.
        """
        supported_networks: Optional[List[Literal["ach", "us_domestic_wire"]]]
        """
        The list of networks that the address supports
        """
        type: Literal["aba"]
        """
        The type of financial address
        """
        _inner_class_types = {"aba": Aba}

    class PlatformRestrictions(StripeObject):
        inbound_flows: Optional[Literal["restricted", "unrestricted"]]
        """
        Restricts all inbound money movement.
        """
        outbound_flows: Optional[Literal["restricted", "unrestricted"]]
        """
        Restricts all outbound money movement.
        """

    class StatusDetails(StripeObject):
        class Closed(StripeObject):
            reasons: List[
                Literal["account_rejected", "closed_by_platform", "other"]
            ]
            """
            The array that contains reasons for a FinancialAccount closure.
            """

        closed: Optional[Closed]
        """
        Details related to the closure of this FinancialAccount
        """
        _inner_class_types = {"closed": Closed}

    if TYPE_CHECKING:

        class CreateParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            features: NotRequired["FinancialAccount.CreateParamsFeatures|None"]
            """
            Encodes whether a FinancialAccount has access to a particular feature. Stripe or the platform can control features via the requested field.
            """
            metadata: NotRequired["Dict[str, str]|None"]
            """
            Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
            """
            platform_restrictions: NotRequired[
                "FinancialAccount.CreateParamsPlatformRestrictions|None"
            ]
            """
            The set of functionalities that the platform can restrict on the FinancialAccount.
            """
            supported_currencies: List[str]
            """
            The currencies the FinancialAccount can hold a balance in.
            """

        class CreateParamsPlatformRestrictions(TypedDict):
            inbound_flows: NotRequired[
                "Literal['restricted', 'unrestricted']|None"
            ]
            """
            Restricts all inbound money movement.
            """
            outbound_flows: NotRequired[
                "Literal['restricted', 'unrestricted']|None"
            ]
            """
            Restricts all outbound money movement.
            """

        class CreateParamsFeatures(TypedDict):
            card_issuing: NotRequired[
                "FinancialAccount.CreateParamsFeaturesCardIssuing|None"
            ]
            """
            Encodes the FinancialAccount's ability to be used with the Issuing product, including attaching cards to and drawing funds from the FinancialAccount.
            """
            deposit_insurance: NotRequired[
                "FinancialAccount.CreateParamsFeaturesDepositInsurance|None"
            ]
            """
            Represents whether this FinancialAccount is eligible for deposit insurance. Various factors determine the insurance amount.
            """
            financial_addresses: NotRequired[
                "FinancialAccount.CreateParamsFeaturesFinancialAddresses|None"
            ]
            """
            Contains Features that add FinancialAddresses to the FinancialAccount.
            """
            inbound_transfers: NotRequired[
                "FinancialAccount.CreateParamsFeaturesInboundTransfers|None"
            ]
            """
            Contains settings related to adding funds to a FinancialAccount from another Account with the same owner.
            """
            intra_stripe_flows: NotRequired[
                "FinancialAccount.CreateParamsFeaturesIntraStripeFlows|None"
            ]
            """
            Represents the ability for the FinancialAccount to send money to, or receive money from other FinancialAccounts (for example, via OutboundPayment).
            """
            outbound_payments: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundPayments|None"
            ]
            """
            Includes Features related to initiating money movement out of the FinancialAccount to someone else's bucket of money.
            """
            outbound_transfers: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundTransfers|None"
            ]
            """
            Contains a Feature and settings related to moving money out of the FinancialAccount into another Account with the same owner.
            """

        class CreateParamsFeaturesOutboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundTransfersAch|None"
            ]
            """
            Enables ACH transfers via the OutboundTransfers API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundTransfersUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundTransfers API.
            """

        class CreateParamsFeaturesOutboundTransfersUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesOutboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesOutboundPayments(TypedDict):
            ach: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundPaymentsAch|None"
            ]
            """
            Enables ACH transfers via the OutboundPayments API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.CreateParamsFeaturesOutboundPaymentsUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundPayments API.
            """

        class CreateParamsFeaturesOutboundPaymentsUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesOutboundPaymentsAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesIntraStripeFlows(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesInboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.CreateParamsFeaturesInboundTransfersAch|None"
            ]
            """
            Enables ACH Debits via the InboundTransfers API.
            """

        class CreateParamsFeaturesInboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesFinancialAddresses(TypedDict):
            aba: NotRequired[
                "FinancialAccount.CreateParamsFeaturesFinancialAddressesAba|None"
            ]
            """
            Adds an ABA FinancialAddress to the FinancialAccount.
            """

        class CreateParamsFeaturesFinancialAddressesAba(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesDepositInsurance(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class CreateParamsFeaturesCardIssuing(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ListParams(RequestOptions):
            created: NotRequired["FinancialAccount.ListParamsCreated|int|None"]
            ending_before: NotRequired["str|None"]
            """
            An object ID cursor for use in pagination.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            limit: NotRequired["int|None"]
            """
            A limit ranging from 1 to 100 (defaults to 10).
            """
            starting_after: NotRequired["str|None"]
            """
            An object ID cursor for use in pagination.
            """

        class ListParamsCreated(TypedDict):
            gt: NotRequired["int|None"]
            """
            Minimum value to filter by (exclusive)
            """
            gte: NotRequired["int|None"]
            """
            Minimum value to filter by (inclusive)
            """
            lt: NotRequired["int|None"]
            """
            Maximum value to filter by (exclusive)
            """
            lte: NotRequired["int|None"]
            """
            Maximum value to filter by (inclusive)
            """

        class ModifyParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            features: NotRequired["FinancialAccount.ModifyParamsFeatures|None"]
            """
            Encodes whether a FinancialAccount has access to a particular feature, with a status enum and associated `status_details`. Stripe or the platform may control features via the requested field.
            """
            metadata: NotRequired["Dict[str, str]|None"]
            """
            Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
            """
            platform_restrictions: NotRequired[
                "FinancialAccount.ModifyParamsPlatformRestrictions|None"
            ]
            """
            The set of functionalities that the platform can restrict on the FinancialAccount.
            """

        class ModifyParamsPlatformRestrictions(TypedDict):
            inbound_flows: NotRequired[
                "Literal['restricted', 'unrestricted']|None"
            ]
            """
            Restricts all inbound money movement.
            """
            outbound_flows: NotRequired[
                "Literal['restricted', 'unrestricted']|None"
            ]
            """
            Restricts all outbound money movement.
            """

        class ModifyParamsFeatures(TypedDict):
            card_issuing: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesCardIssuing|None"
            ]
            """
            Encodes the FinancialAccount's ability to be used with the Issuing product, including attaching cards to and drawing funds from the FinancialAccount.
            """
            deposit_insurance: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesDepositInsurance|None"
            ]
            """
            Represents whether this FinancialAccount is eligible for deposit insurance. Various factors determine the insurance amount.
            """
            financial_addresses: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesFinancialAddresses|None"
            ]
            """
            Contains Features that add FinancialAddresses to the FinancialAccount.
            """
            inbound_transfers: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesInboundTransfers|None"
            ]
            """
            Contains settings related to adding funds to a FinancialAccount from another Account with the same owner.
            """
            intra_stripe_flows: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesIntraStripeFlows|None"
            ]
            """
            Represents the ability for the FinancialAccount to send money to, or receive money from other FinancialAccounts (for example, via OutboundPayment).
            """
            outbound_payments: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundPayments|None"
            ]
            """
            Includes Features related to initiating money movement out of the FinancialAccount to someone else's bucket of money.
            """
            outbound_transfers: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundTransfers|None"
            ]
            """
            Contains a Feature and settings related to moving money out of the FinancialAccount into another Account with the same owner.
            """

        class ModifyParamsFeaturesOutboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundTransfersAch|None"
            ]
            """
            Enables ACH transfers via the OutboundTransfers API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundTransfersUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundTransfers API.
            """

        class ModifyParamsFeaturesOutboundTransfersUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesOutboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesOutboundPayments(TypedDict):
            ach: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundPaymentsAch|None"
            ]
            """
            Enables ACH transfers via the OutboundPayments API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesOutboundPaymentsUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundPayments API.
            """

        class ModifyParamsFeaturesOutboundPaymentsUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesOutboundPaymentsAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesIntraStripeFlows(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesInboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesInboundTransfersAch|None"
            ]
            """
            Enables ACH Debits via the InboundTransfers API.
            """

        class ModifyParamsFeaturesInboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesFinancialAddresses(TypedDict):
            aba: NotRequired[
                "FinancialAccount.ModifyParamsFeaturesFinancialAddressesAba|None"
            ]
            """
            Adds an ABA FinancialAddress to the FinancialAccount.
            """

        class ModifyParamsFeaturesFinancialAddressesAba(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesDepositInsurance(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class ModifyParamsFeaturesCardIssuing(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class RetrieveParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """

        class RetrieveFeaturesParams(RequestOptions):
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """

        class UpdateFeaturesParams(RequestOptions):
            card_issuing: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsCardIssuing|None"
            ]
            """
            Encodes the FinancialAccount's ability to be used with the Issuing product, including attaching cards to and drawing funds from the FinancialAccount.
            """
            deposit_insurance: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsDepositInsurance|None"
            ]
            """
            Represents whether this FinancialAccount is eligible for deposit insurance. Various factors determine the insurance amount.
            """
            expand: NotRequired["List[str]|None"]
            """
            Specifies which fields in the response should be expanded.
            """
            financial_addresses: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsFinancialAddresses|None"
            ]
            """
            Contains Features that add FinancialAddresses to the FinancialAccount.
            """
            inbound_transfers: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsInboundTransfers|None"
            ]
            """
            Contains settings related to adding funds to a FinancialAccount from another Account with the same owner.
            """
            intra_stripe_flows: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsIntraStripeFlows|None"
            ]
            """
            Represents the ability for the FinancialAccount to send money to, or receive money from other FinancialAccounts (for example, via OutboundPayment).
            """
            outbound_payments: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundPayments|None"
            ]
            """
            Includes Features related to initiating money movement out of the FinancialAccount to someone else's bucket of money.
            """
            outbound_transfers: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundTransfers|None"
            ]
            """
            Contains a Feature and settings related to moving money out of the FinancialAccount into another Account with the same owner.
            """

        class UpdateFeaturesParamsOutboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundTransfersAch|None"
            ]
            """
            Enables ACH transfers via the OutboundTransfers API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundTransfersUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundTransfers API.
            """

        class UpdateFeaturesParamsOutboundTransfersUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsOutboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsOutboundPayments(TypedDict):
            ach: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundPaymentsAch|None"
            ]
            """
            Enables ACH transfers via the OutboundPayments API.
            """
            us_domestic_wire: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsOutboundPaymentsUsDomesticWire|None"
            ]
            """
            Enables US domestic wire transfers via the OutboundPayments API.
            """

        class UpdateFeaturesParamsOutboundPaymentsUsDomesticWire(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsOutboundPaymentsAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsIntraStripeFlows(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsInboundTransfers(TypedDict):
            ach: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsInboundTransfersAch|None"
            ]
            """
            Enables ACH Debits via the InboundTransfers API.
            """

        class UpdateFeaturesParamsInboundTransfersAch(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsFinancialAddresses(TypedDict):
            aba: NotRequired[
                "FinancialAccount.UpdateFeaturesParamsFinancialAddressesAba|None"
            ]
            """
            Adds an ABA FinancialAddress to the FinancialAccount.
            """

        class UpdateFeaturesParamsFinancialAddressesAba(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsDepositInsurance(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

        class UpdateFeaturesParamsCardIssuing(TypedDict):
            requested: bool
            """
            Whether the FinancialAccount should have the Feature.
            """

    active_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to active Features in the Features hash.
    """
    balance: Balance
    """
    Balance information for the FinancialAccount
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    features: Optional["FinancialAccountFeatures"]
    """
    Encodes whether a FinancialAccount has access to a particular Feature, with a `status` enum and associated `status_details`.
    Stripe or the platform can control Features via the requested field.
    """
    financial_addresses: List[FinancialAddress]
    """
    The set of credentials that resolve to a FinancialAccount.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["treasury.financial_account"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    pending_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to pending Features in the Features hash.
    """
    platform_restrictions: Optional[PlatformRestrictions]
    """
    The set of functionalities that the platform can restrict on the FinancialAccount.
    """
    restricted_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to restricted Features in the Features hash.
    """
    status: Literal["closed", "open"]
    """
    The enum specifying what state the account is in.
    """
    status_details: StatusDetails
    supported_currencies: List[str]
    """
    The currencies the FinancialAccount can hold a balance in. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase.
    """

    @classmethod
    def create(
        cls,
        api_key: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.CreateParams"]
    ) -> "FinancialAccount":
        """
        Creates a new FinancialAccount. For now, each connected account can only have one FinancialAccount.
        """
        return cast(
            "FinancialAccount",
            cls._static_request(
                "post",
                cls.class_url(),
                api_key,
                idempotency_key,
                stripe_version,
                stripe_account,
                params,
            ),
        )

    @classmethod
    def list(
        cls,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.ListParams"]
    ) -> ListObject["FinancialAccount"]:
        """
        Returns a list of FinancialAccounts.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["FinancialAccount.ModifyParams"]
    ) -> "FinancialAccount":
        """
        Updates the details of a FinancialAccount.
        """
        url = "%s/%s" % (cls.class_url(), quote_plus(id))
        return cast(
            "FinancialAccount",
            cls._static_request("post", url, params=params),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["FinancialAccount.RetrieveParams"]
    ) -> "FinancialAccount":
        """
        Retrieves the details of a FinancialAccount.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    def _cls_retrieve_features(
        cls,
        financial_account: str,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.RetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            cls._static_request(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=util.sanitize_id(financial_account)
                ),
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def retrieve_features(
        financial_account: str,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.RetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @overload
    def retrieve_features(
        self,
        idempotency_key: Optional[str] = None,
        **params: Unpack["FinancialAccount.RetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @class_method_variant("_cls_retrieve_features")
    def retrieve_features(  # pyright: ignore[reportGeneralTypeIssues]
        self,
        idempotency_key: Optional[str] = None,
        **params: Unpack["FinancialAccount.RetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=util.sanitize_id(self.get("id"))
                ),
                idempotency_key=idempotency_key,
                params=params,
            ),
        )

    @classmethod
    def _cls_update_features(
        cls,
        financial_account: str,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.UpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            cls._static_request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=util.sanitize_id(financial_account)
                ),
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def update_features(
        financial_account: str,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack["FinancialAccount.UpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @overload
    def update_features(
        self,
        idempotency_key: Optional[str] = None,
        **params: Unpack["FinancialAccount.UpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @class_method_variant("_cls_update_features")
    def update_features(  # pyright: ignore[reportGeneralTypeIssues]
        self,
        idempotency_key: Optional[str] = None,
        **params: Unpack["FinancialAccount.UpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=util.sanitize_id(self.get("id"))
                ),
                idempotency_key=idempotency_key,
                params=params,
            ),
        )

    _inner_class_types = {
        "balance": Balance,
        "financial_addresses": FinancialAddress,
        "platform_restrictions": PlatformRestrictions,
        "status_details": StatusDetails,
    }
